package vishnu.builder;

import java.util.*;
import java.io.*;

public class CollectionBuilder
{
    //String site;
    String collID = null; // directory name
    String collHome = null; // full path to collections directory
    String collName = null; // descriptive name
    String collDesc = null; // descriptive description :-)
    String indexer = null; // mg or lucene
    boolean use_views = false;
    long numDocs=0;
    Vector views = null;
    private static void usage()
    {
	System.out.println("java vishnu.builder.CollectionBuilder\n -collsHome (directory where collections live) \n -collection (directory name of collection)\n -name (descriptive name for collection) \n -description (collection description)\n -index (\"mg\" or \"luc\")\n [-use_views ]");
    }
    
    
    public static void main(String[] args)
    {
	if( args.length == 0 ){
	    usage();
	    System.exit(0);
	}
	
	String collID = null;
	String collsHome = null;
	String collName = "";
	String collDesc = "";
	String indexer = null;
	boolean use_views = false;
	int a = 0;
	while( a < args.length ){
	    if (args[a].equals("-collsHome"))
		collsHome = args[++a];
	    else if (args[a].equals("-collection"))
		collID = args[++a];
	    else if( args[a].equals("-name") )
		collName = args[++a];
	    else if( args[a].equals("-description") )
		collDesc = args[++a];
	    else if( args[a].equals("-index") )
		indexer = args[++a];
	    else if (args[a].equals("-use_views")) 
		use_views = true;
	    a++;
	}
	
	if (collsHome==null || collID==null || indexer==null) {
	    usage();
	    System.exit(0);
	}
	
	if (collName.equals("")) {
	    collName = collID;
	}
	if( !indexer.equals("mg") && !indexer.equals("luc") )
	    indexer = "luc";
	
	if (!collsHome.endsWith(File.separator)) {
	    collsHome += File.separator;
	}
	String coll_home = collsHome + collID + File.separator;
	
	CollectionBuilder cb = new CollectionBuilder(coll_home, collID, collName, collDesc, indexer, use_views);
    }
    
    
    
    public CollectionBuilder(String coll_home, String coll_id, String coll_name,String coll_desc,  String ind, boolean use_views)
    {
	this.collHome = coll_home;
	this.collID = coll_id;
	this.collName = coll_name;
	this.collDesc = coll_desc;
	this.indexer = ind;
	this.use_views = use_views;
	
	if (use_views) {
	    views = new Vector();
	}
	if (!setupCollection()) {
	    System.err.println("Couldn't setup the collection. Quitting build");
	    return;
	}
	
	/**** get text annotation ****/

	FTIndexing();


	/**** extract keywords ****/

	CKIndexing();

	/**** write out the config file ****/
	printConfig();
    }



    private boolean setupCollection()
    {
	// check the import directory
	File import_dir = new File(collHome + "import");
	if (!import_dir.isDirectory()) {
	    System.err.println("the import directory "+import_dir.getPath()+" doesn't exist. Quitting build");
	    return false;
	}
	
	
	// create the building directory
	try {
	    File index_dir = new File(collHome + "building");
	    if (!index_dir.exists()) {
		index_dir.mkdir();
	    } else if (!index_dir.isDirectory()) {
		// index is present but is not a directory
		System.err.println("Trying to create building dir but a file already exists at "+index_dir.getPath());
		return false;
	    }
	} catch (Exception e) {
	    System.err.println("CollectionBuilder.setupCollection Error: "+e);
	    return false;
	}

	// create the view list
	if (use_views) {
	    // create a new view for each sub directory in import
	    File [] sub_dirs = import_dir.listFiles();
	    for (int i=0; i<sub_dirs.length; i++) {
		if (sub_dirs[i].isDirectory()) {
		    views.add(sub_dirs[i].getName());
		}
	    }
	    if (views.size()==0) {
		// we haven't found any subdirectories
		System.err.println("no sub directories were found in the import dorectory ("+import_dir.getPath()+"), not using views");
		use_views = false;

	    }
	}
	return true;
    }

    private void printConfig() {

	String index_type="";
	if (indexer.equals("luc")) {
	    index_type = "LUCENE";
	} else if (indexer.equals("mg")) {
	    index_type = "MG";
	}
	
	try{
	    File target = new File(collHome + "config.xml");
	    PrintWriter out = new PrintWriter(new FileWriter(target),true);
	
	    StringBuffer buffer = new StringBuffer();
	    buffer.append("<collection name=\"");
	    buffer.append(collName);
	    buffer.append("\" description=\"");
	    buffer.append(collDesc);
	    buffer.append("\" engine=\"");
	    buffer.append(index_type);
	    buffer.append("\" size=\"");
	    buffer.append(String.valueOf(numDocs));
	    buffer.append("\">\n");
	    
	    // add in views
	    if (use_views) {
		buffer.append("<viewList>\n");
		for (int i=0; i<views.size(); i++) {
		    buffer.append("<view name=\"");
		    buffer.append(views.get(i));
		    buffer.append("\"/>");
		}
		buffer.append("\n</viewList>\n");
	    } 
	    buffer.append("</collection>\n");
		
	    out.print(buffer.toString());
	    out.close();

	} catch (IOException e){ System.out.println("Error writing config file: " + e.toString());
	}
    }
    

    private void CKIndexing()
    {
	CKIndexer cki = new CKIndexer();
	cki.setCollectionName(collID);
	String in = collHome + "import"+File.separator;
	String out = collHome + "building"+File.separator+"ck_index"+File.separator;
	if (use_views) {
	    for (int i=0; i<views.size(); i++) {
		String view = (String)views.get(i);
		System.out.println("calculating candidate keywords for view "+view);
		cki.setOutputDirectory(out+view+File.separator);
		cki.setInputDirectory(in+view+File.separator);
		cki.startIndexing();
	    }
	} else {
	    System.out.println("calculating candidate keywords");
	    // just do the import dir as a whole
	    cki.setOutputDirectory(out);
	    cki.setInputDirectory(in);
	    cki.startIndexing();
	}
    }
    


    private void FTIndexing()
    {
	String in = collHome + "import"+File.separator;
	String out = collHome + "building"+File.separator + indexer + "_index"+File.separator;
	Indexer the_indexer;
	if (indexer.equals("luc") ) {
	    the_indexer = new LuceneWrapper();
	} else {
	    the_indexer = new MGWrapper();
	}
	the_indexer.setCollectionName(collID);

	if (use_views) {
	    for (int i=0; i<views.size(); i++) {
		String view = (String)views.get(i);
		System.out.println("indexing the text for view "+view);
		the_indexer.setOutputDirectory(out+view+File.separator);
		the_indexer.setInputDirectory(in+view+File.separator);
		the_indexer.startIndexing();
	    }

	} else {
	    System.out.println("indexing the text");
	    the_indexer.setOutputDirectory(out);
 	    the_indexer.setInputDirectory(in);
 	    the_indexer.startIndexing();
	}

    }
}




